[> Documentation-Index](index.md) [> Setup Guide](setup.md)

# smartClientCore

## Global Config
- [debug](#debug)
- [blockInsecureURL](#blockinsecureurl)
- [maxWrapperDepth](#maxwrapperdepth)
- [adRequestMethod](#adrequestmethod)
- [adReinsertion](#adreinsertion)
  - [HOMAD](#adreinsertion+homad)
- [adVerification](#adverification)
  - [OMID](#adverification+omid)
  - [nonOMID](#adverification+nonomid)
- [skipOffset](#skipoffset)
- [timeouts](#timeouts)
  - [adRequest](#timeouts+adrequest)
  - [adRequestChain](#timeouts+adrequestchain)
  - [adSlotRequest](#timeouts+adslotrequest)
  - [videoRequest](#timeouts+videorequest)
  - [vpaidRequest](#timeouts+vpaidrequest)
  - [vpaidReadiness](#timeouts+vpaidreadiness)
  - [nonlinearMaxDuration](#timeouts+nonlinearmaxduration)

<a name="debug"></a>

### debug : <code>Boolean</code>
Enables log output to the console from the start.
In addition, the log output can be activated at runtime by calling `localStorage.setItem ('debug', true);`.

<a name="blockinsecureurl"></a>

### blockInsecureURL : <code>Boolean</code>
By default insecure URL requests (tags, tracker, media files) will not be executed within secure environment. Default is `true`.

<a name="maxwrapperdepth"></a>

### maxWrapperDepth : <code>number</code>
When serving an ad involves a chain of wrappers, an infinite loop is possible where a chain of Wrappers never results in a final inline VAST response. 
In general, wrappers should be limited to ten before resulting in an inline response. If the add-on detects more than ten wrappers,
the add-on rejects any subsequent responses in the chain and returns VAST error 302.
Default limit is `10` wrappers.

<a name="adrequestmethod"></a>

### adRequestMethod : <code>string</code>
By default, ads are requested through **XHR**. This may not work with applications which target devices that do not support CORS 
or have not implemented the `XMLHTTPRequest` object, especially HbbTV apps for SmartTVs. As an alternative, ads can be requested through **JSONP**.  
Possible values are `smartclientcore.AD_REQUEST_METHOD.XHR` (the default if omitted) and `smartclientcore.AD_REQUEST_METHOD.JSONP`.

<a name="adreinsertion"></a>

### adReinsertion : <code>Object</code>

<a name="adreinsertion+homad"></a>

#### homad : <code>Object</code>
The add-on has been extended with the **HOMAD HTML5 solution** to help bypassing ad-blockers.
All ad-related resources (tags, creative files, trackers) will be redirected to HOMAD once they get blocked for any reason.

**enabled : <code>Boolean</code>** - Enables HOMAD ad-reinsertion. Disabled by default.

**setup : <code>Object</code>** - HOMAD configuration

| Param | Type | Description |
| --- | --- | --- |
| globalConfig | <code>string</code> | URI to HOMAD's global configuration file. |
| clientConfig | <code>string</code> | URI to the client-specific configuration file. |
| enableSessionCoverage | <code>Boolean</code> | Decide whether ad-blockers should be detected again when requesting a new ad-break (default behavior) or not.<br><br>Disabled by default. |
| activationCallback([reason <code>number</code>](#adreinsertion+activation-codes)) | ⇒ <code>Promise</code> | Is called whenever the add-on detects a scenario that blocks or would block an ad-related request.<br>Based on the specified [reason](#adreinsertion+activation-codes) the player has to decide whether the current ad-slot must be requested again through HOMAD or not.<br><br>Resolves if the publisher decides to re-call the current ad-break through HOMAD. |

> _If the `activationCallback` is omitted or does not respond within 250 milliseconds, 
> HOMAD will be automatically activated (if `enabled` before)._


```javascript
let globalAdConfig = smartclientcore.SCHEMA.create(smartclientcore.SCHEMA.GLOBAL_CONFIG);
globalAdConfig.adReinsertion = {
	homad: {
		enabled: true,
		setup: {
			globalConfig: 'https://s3.amazonaws.com/homad-global-configs.schneevonmorgen.com/global_config.json',
			clientConfig: 'https://test.s3.amazonaws.com/homad/homadConfigTestHttps.json',
			enableSessionCoverage: true,
			activationCallback: (reason) => {
				return new Promise((resolve, reject) => {
					
					// The reason for activating HOMAD sounds reasonable?
					resolve();
					
					// otherwise
					//reject();
				});
			}
		}
	}
};
```

<a name="adreinsertion+activation-codes"></a>

#### Activation Reason - Codes Table

| Code | ENUM | Description |
| --- | --- | --- |
| 1000 | `AdReinsertionModel.REASON.XHR_TIMEOUT` | Timeout of ad-request URIs |
| 1001 | `AdReinsertionModel.REASON.XHR_UNEXPECTED_STATUS` | XHR request status is not 200 (ok)  |
| 1004 | `AdReinsertionModel.REASON.WRAPPER_CHAIN_TIMEOUT` | Timeout of wrapper request chain  |
| 1010 | `AdReinsertionModel.REASON.MEDIAFILE_TIMEOUT` | Timeout of media file request URIs |
| 1013 | `AdReinsertionModel.REASON.MEDIAFILE_LOAD_ERROR` | General load error of media file URIs |


<a name="adverification"></a>

### adVerification : <code>Object</code>

<a name="adverification+omid"></a>

#### OMID : <code>Object</code>
Ad-verification vendors that rely on the Open Measurement Interface Definition API (**OMID**) are enabled by default.
The following properties may be adjusted to enable selected verification vendors only and to limit the measurement session execution time.

More information on OMID and OM SDK available at: [IAB]

**vendorWhitelist : <code>[ &#x27;Array&#x27; ].&lt;string&gt;</code>** - List of accepted AdVerification vendors (whitelist).
Each specified String has to represent an unique identifier that can be found within the verification code resource URI, 
most likely the domain name.

If omitted, the add-on tries to load verification code from each vendor.

**sessionCleanupDelay : <code>number</code>** - Time value in seconds that specifies a delay for closing the current measurement session
after the ad has finished playback. 
Default is `3` seconds.

> _The verification code should have sufficient time to ensure any measurement 
and reporting is completed before cleaning up, in order to minimize discrepancies._ 

**timeoutScriptLoad : <code>number</code>** - Representing the number of seconds a OMID vendor can take to load and initialize its code before being terminated.
Defaulting to `0.5` seconds if omitted.

**playerHandles : <code>Boolean</code>** - Whether the player takes responsibility for Open Measurement, e.g. native implementations of OMID. 
Defaulting to `false` if omitted.


<a name="adverification+nonomid"></a>

#### nonOMID : <code>Object</code>
Ad-verification vendors that base or rely on proprietary measurement and reporting logic.

**meetrics : <code>Boolean</code>** - Disabled by default.
Enables Meetrics viewability measurement for video advertisment with any VAST and VPAID creative.


<a name="skipoffset"></a>

### skipOffset : <code>number</code>
Time value in seconds that identifies when skip controls are made available to the end user by default.
Advertisers may define a `skipoffset` value in its VAST tag that has higher priority than the publisher setting.


<a name="timeouts"></a>

### timeouts : <code>Object</code>
```javascript
let globalAdConfig = smartclientcore.SCHEMA.create(smartclientcore.SCHEMA.GLOBAL_CONFIG);
globalAdConfig.timeouts = {
	adRequest: 1,
	adRequestChain: 2,
	adSlotRequest: 4,
	videoRequest: 3,
	vpaidRequest: 1,
	vpaidReadiness: 3,
	nonlinearMaxDuration: 10
};
```

<a name="timeouts+adrequest"></a>

#### adRequest : <code>number</code>
The number of seconds a single ad request can take to load before being terminated.  
Default is `1` seconds.
 
<a name="timeouts+adrequestchain"></a>

#### adRequestChain : <code>number</code>
The total number of seconds a single ad request chain (VAST Wrappers and final VAST Inline tag) can take to load before being terminated.  
Default is `2` seconds.

<a name="timeouts+adslotrequest"></a>

#### adSlotRequest : <code>number</code>
The total number of seconds an ad-slot request (from calling `initAdSlot` until the Promise resolves or rejects) can take before being terminated.  
Default is `4` seconds.

<a name="timeouts+videorequest"></a>

#### videoRequest : <code>number</code>
Representing the number of seconds a video creative can take to load before being terminated.  
Default is `3` seconds.

<a name="timeouts+vpaidrequest"></a>

#### vpaidRequest : <code>number</code>
Representing the number of seconds a VPAID creative (JavaScript resource) can take to load before being terminated.  
Default is `1` seconds.

<a name="timeouts+vpaidreadiness"></a>

#### vpaidReadiness : <code>number</code>
Representing the number of seconds a VPAID creative can take to initialize and respond 
(from calling `VPAID.initAd()` until the creative triggers the `AdLoaded` event) before being terminated.  
Default is `3` seconds.

<a name="timeouts+nonlinearmaxduration"></a>

#### nonlinearMaxDuration : <code>number</code>
The maximum time in seconds a NonLinear ad is displayed (from triggering the impression) before being terminated.
**_Note_**_: The timer becomes paused for the time a creative has entered the linear state._   
Default is `10` seconds.

[IAB]:(https://www.iab.com/omsdk)
